#ifndef _BRIDGE_ATTR_H
#define _BRIDGE_ATTR_H

#include <tango.h>
#include <stdio.h>
#include <stdlib.h>
#include <RPC2TangoBridge.h>
#include <RpcReadHandle.h>
#include <RpcWriteHandle.h>

using namespace Tango;

namespace RPC2TangoBridge_ns
{

class BridgeAttr : public Tango::Attr
{
	public:
		/** Constructor for Tango::READ attributes (read only )
		 */
		BridgeAttr(const char *name,long data_type,Tango::AttrWriteType w_type);
		
		/** Constructor for Tango::READ_WRITE attributes (scalar only). Wants a writeName as parameter, 
		 * that indicates the RPC call to execute when writing attribute.
		 */
		BridgeAttr(const char *name, const char* writeName, long data_type,Tango::AttrWriteType w_type);

		~BridgeAttr() ;
		virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
			{(static_cast<RPC2TangoBridge *>(dev))->read_BridgeAttr(att);}
		virtual void write(Tango::DeviceImpl *dev,Tango::WAttribute &att)
			{(static_cast<RPC2TangoBridge *>(dev))->write_BridgeAttr(att);}
		virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
			{return (static_cast<RPC2TangoBridge *>(dev))->is_BridgeAttr_allowed(ty);}

		RPCHandle *rpcHandle() { return rpch; }
		
		std::string host();
		unsigned short port();
		
		bool connected() { return d_connected; }
		
		void setRpcWriteHandle(RPCWriteHandle *h) { wrpch = h; }
		RPCWriteHandle *rpcWriteHandle() { return wrpch; }
		
		RPCReadHandle *rpcReadHandle() { return rrpch; }
		
		string writeName() { return d_writeName; }

	private:
		RPCHandle *rpch;
		RPCWriteHandle *wrpch;
		RPCReadHandle *rrpch;
		std::string d_writeName;
		bool d_connected;
};

class SpectrumBridgeAttr : public Tango::SpectrumAttr
{
	public:
		SpectrumBridgeAttr(const char *name,long data_type,Tango::AttrWriteType w_type, long max_x);
		~SpectrumBridgeAttr();

		 virtual void read(Tango::DeviceImpl *dev,Tango::Attribute &att)
                        {(static_cast<RPC2TangoBridge *>(dev))->read_BridgeAttr(att);}
                virtual bool is_allowed(Tango::DeviceImpl *dev,Tango::AttReqType ty)
                        {return (static_cast<RPC2TangoBridge *>(dev))->is_BridgeAttr_allowed(ty);}

		RPCHandle *rpcHandle() { return rpch; }
		RPCReadHandle *rpcReadHandle() { return rrpch; }
		bool connected() { return d_connected; }
			
		string host();
		unsigned short port();
		
        private:
                RPCHandle *rpch;
		RPCWriteHandle *wrpch;
		RPCReadHandle *rrpch;
		bool d_connected;
};


} /* namespace RPC2TangoBridge_ns */
#endif

