#include <RpcWriteHandle.h>
#include <RpcResolver.h>
#include <errtype.h>

#define MAXERRLEN	512

using namespace std;

namespace RPC2TangoBridge_ns
{

static omni_mutex rpcMutex;

RPCWriteHandle::RPCWriteHandle(RPCHandle *h) : RPCHandle(h)
{
	
}

RPCWriteHandle::~RPCWriteHandle()
{
	
}

bool RPCWriteHandle::write_long(const char *callname, idl_long_int lvalue)
{
	bool ret;
	char error[MAXERRLEN];
	long res = -1;
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];

	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
		return false;
	}
//	printf("\e[1;32mWRITE\e[0m scrivo %ld su %s %s %s %s\n",lvalue,  fam, mem, act, mode);
	
	/* acquire lock! */
	lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "WI") || !strcmp(mode, "WII"))
			bind_lpc(_sid);
		if(!strcmp(mode, "WI"))
			res = lpc_WI(fam, mem, act, mode, &lvalue);
		else if(!strcmp(mode, "WII"))
				res = lpc_WII(fam, mem, act, mode, &lvalue);
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error writeing rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
		}
	}
	/* release lock */
	unlockRpc();
        return ret;

}

bool RPCWriteHandle::write_double(const char *callname,  double dvalue)
{
	bool ret;
	long res = -1;
	char error[MAXERRLEN];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];

	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
		return false;
	}
//	printf("\e[1;32mWRITE\e[0m scrivo double %lf su %s %s %s %s\n",dvalue,  fam, mem, act, mode);

	/* acquire lock! */
	lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		ret = false;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "WR") || !strcmp(mode, "WRI"))
			bind_lpc(_sid);
		if(!strcmp(mode, "WR"))
			res = lpc_WR(fam, mem, act, mode, &dvalue);
		else if(!strcmp(mode, "WRI"))
				res = lpc_WRI(fam, mem, act, mode, &dvalue);
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error writing rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 ret = false;
		}
		else 
			ret = true;
	}
	/* release lock */
	unlockRpc();
        return ret;
}

bool RPCWriteHandle::write_short(const char *callname, idl_short_int svalue)
{
	bool ret;
	long res = -1;
	char error[MAXERRLEN];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	
	/* acquire lock! */
	lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		ret = false;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
	//	printf("\e[1;32mWRITE\e[0m scrivo SHORT %d su %s %s %s %s\n", svalue,  fam, mem, act, mode);
		if(!strcmp(mode, "WS") || !strcmp(mode, "WSI"))
			bind_lpc(_sid);
		if(!strcmp(mode, "WS"))
			res = lpc_WS(fam, mem, act, mode, &svalue);
		else if(!strcmp(mode, "WSI"))
				res = lpc_WSI(fam, mem, act, mode, &svalue);
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error writing rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 ret = false;
		}
		else 
			ret = true;
	}
	/* release lock */
	unlockRpc();
        return ret;
}

bool RPCWriteHandle::write_string(const char *callname, char **s)
{
//	bool ret;
// 	long res = -1;
// 	char error[MAXERRLEN];
// 	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
// 
// 	/* acquire lock! */
// 	lockRpc();
// 	if(split_rpccall(callname, fam, mem, act, mode) < 0)
// 	{
// 		ret = false;
// 		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
// 	}
// 	else
// 	{
// 		if(!strcmp(mode, "WR") || !strcmp(mode, "WRI"))
// 			bind_lpc(_sid);
// 		if(!strcmp(mode, "WR"))
// 			res = lpc_WR(fam, mem, act, mode, &dvalue);
// 		else if(!strcmp(mode, "WRI"))
// 				res = lpc_WRI(fam, mem, act, mode, &dvalue);
// 		
// 		if(res < 0)
// 		{
// 			snprintf(error, MAXERRLEN, "Error writing rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
// 				callname, errno, errtype, errstring);
// 			 _lastRpcErr = string(error);
// 			 ret = false;
// 		}
// 		else 
// 			ret = true;
// 	}
// 	/* release lock */
// 	unlockRpc();
	//printf("\e[1;35m;4minfo\e[0m write_string not implemented yet\n");
//	ret  = -1;
        return false;

}

bool RPCWriteHandle::write_boolean(const char *callname, bool bvalue)
{
	bool ret;
	long res = -1;
	char error[MAXERRLEN];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	unsigned char boolval;
	if(bvalue)
		boolval = 1;
	else
		boolval = 0;

	/* acquire lock! */
	lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		ret = false;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "WB") || !strcmp(mode, "WBI"))
			bind_lpc(_sid);
		if(!strcmp(mode, "WB"))
			res = lpc_WB(fam, mem, act, mode, &boolval);
		else if(!strcmp(mode, "WBI"))
				res = lpc_WBI(fam, mem, act, mode, &boolval);
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error writing rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 ret = false;
		}
		else 
			ret = true;
	}
	/* release lock */
	unlockRpc();
        return ret;
}


bool RPCWriteHandle::write_void(const char *callname)
{
	bool ret;
	long res = -1;
	char error[MAXERRLEN];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	unsigned char dummy;

	/* acquire lock! */
	lockRpc();
	if(split_rpccall(callname, fam, mem, act, mode) < 0)
	{
		ret = false;
		_lastRpcErr = string("Failed to split ")  + string(callname) + " into family, member, action, mode";
	}
	else
	{
		if(!strcmp(mode, "WP") || !strcmp(mode, "WPI"))
			bind_lpc(_sid);
		if(!strcmp(mode, "WP"))
			res = lpc_WP(fam, mem, act, mode, &dummy);
		else if(!strcmp(mode, "WBI"))
				res = lpc_WPI(fam, mem, act, mode, &dummy);
		
		if(res < 0)
		{
			snprintf(error, MAXERRLEN, "Error writing rpc \"%s\": errno: %d, errtype: %d, errstring: %s",
				callname, errno, errtype, errstring);
			 _lastRpcErr = string(error);
			 ret = false;
		}
		else 
			ret = true;
	}
	/* release lock */
	unlockRpc();
        return ret;
}

}

