#include <attribute_converter.h>
#include <RpcHandle.h>
#include <tango.h>

#define MAX_X  512

/* RPCSPLITLEN and MAXLEN defined in RpcHandle.h */

using namespace std;
using namespace Tango;

namespace RPC2TangoBridge_ns
{


AttributeConverter::AttributeConverter(char *propertyName, bool force_spectrum)
{
	sc_attr = NULL;
	sp_attr = NULL;
	spectrum = false;
	bool readWrite;
	int data_type;
	char res[256];
	
	/* RPCSPLITLEN and MAXLEN defined in RpcHandle.h */
	char writeCall[MAXLEN];
	char readCall[MAXLEN];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	UserDefaultAttrProp default_att_prop;
	
	//Database *db = new Database();
    //DbData data;
	
	memset(fam, 0, RPCSPLITLEN);
	memset(mem, 0, RPCSPLITLEN);
	memset(act, 0, RPCSPLITLEN);
	memset(mode, 0, RPCSPLITLEN);
	
	/* isReadWrite() always copies _read_ RPC call in readCall.
	* If read write, copies in writeCall the write RPC call and returns true, otherwise it 
	* returns false and strlen(writeCall) is 0.
	*/
	readWrite = isReadWrite(propertyName, readCall, writeCall);
	//printf ("AttributeConverter %s readWrite: %d\n",propertyName,readWrite);
	/* used to determine data_type below. We refer to read name, we are not
	 * interested in write_name. The configuration of the device properties must
	 * be consistent, but we do not check it.
	 */
	if(RPCHandle::split_rpccall(readCall, fam, mem, act, mode) < 0)
	{
		//printf("\e[1;31m*\e[0m Error in RPC split: \"%s\"\n", propertyName);
		exit(EXIT_FAILURE);
	}
	
	data_type = data_type_for_mode(mode);
        //printf("creating in converter attribute for \"%s\": data type: \"%s\"\n", propertyName, dataType(data_type).c_str());
	if(data_type >= 0 && (is_spectrum(mode) ||  force_spectrum))
	{	
		spectrum = true;
		sp_attr = createSpectrumAttribute(propertyName, data_type);
	}
	else if(data_type >= 0)
	{
		spectrum = false;
		if(!readWrite)
			sc_attr = createScalarAttribute(readCall, data_type);
		else
			sc_attr = createScalarAttribute(readCall, writeCall, data_type);
	}
	else if(data_type < 0)
	{
// 		snprintf(res, 256, "\e[1;31m* \e[0merror creating \"%s\""
// 			"\e[0;37m [\e[1;31munrecognized data type \e[0;37m]\e[0m", propertyName);
        snprintf(res, 256, "merror creating %s :unrecognized data type ", propertyName);
		result = string(res);
	}
}

bool AttributeConverter::isSpectrum()
{
	return spectrum;
}

bool AttributeConverter::string1_contains2(const char* s1, const char* s2)
{
	int len1 = strlen(s1);
	int len2 = strlen(s2);
	int i;

	for(i = 0; i <= len1 - len2; i++)
	{
		if(strncmp(s1 + i, s2, len2) == 0)
			return true;
	}
	return false;
}

/* detects if the entry read from the tango attribute properties contains a ':' separator
* and, if so, extracts and saves in rwname the Write part (the RPC WRITE call.
* rwname must be a pointer to a MAXLEN char vector.
* Always copies read RPC call in rname.
* If read write, copies in rwname the write RPC call and returns true, otherwise it 
* returns false and strlen(rwname) is 0
*/
bool AttributeConverter::isReadWrite(char* entry, char * rname, char *rwname)
{
	unsigned int i;
	memset(rname, 0, MAXLEN);
	memset(rwname, 0, MAXLEN);
	for(i = 0; i< MAXLEN - 1 && i < strlen(entry); i++)
		if(entry[i] == ':')
			break;
	if(i == strlen(entry))
	{
		strncpy(rname, entry, MAXLEN - 1);
		return false;
	}
	else
	{
		strncpy(rname, entry , i );
// 		printf("AttributeConverter::isReadWrite read part: %s\n", rname);
		memset(rwname, 0, MAXLEN);
		strncpy(rwname, entry + i + 1, MAXLEN - 1);
// 		printf("AttributeConverter::isReadWrite write part: %s\n", rwname);
		return true;
	}
}

bool AttributeConverter::is_spectrum(const char *mode)
{
	/* Read Array (RA) or Read Flags (RF) means a spectrum */
	if(string1_contains2(mode, "RA") || string1_contains2(mode, "RF"))
		return true;
	else
		return false;
}

long int AttributeConverter::data_type_for_mode(const char *mode)
{
	if(!strcasecmp(mode, "RB") || !strcasecmp(mode, "RBI") ||
		!strcasecmp(mode, "RAB") || !strcasecmp(mode, "RAIB"))
		return Tango::DEV_BOOLEAN;
	
	/* flags type, is dev Boolean */
	else if(!strcasecmp(mode, "RF") || !strcasecmp(mode, "RFI"))
		return Tango::DEV_BOOLEAN;
	
	else if(!strcasecmp(mode, "RI") || !strcasecmp(mode, "RII") ||
		!strcasecmp(mode, "RAI") || !strcasecmp(mode, "RAII"))
		return Tango::DEV_LONG;
	
	else if(!strcasecmp(mode, "RR") || !strcasecmp(mode, "RRI") ||
		!strcasecmp(mode, "RAR") || !strcasecmp(mode, "RAIR"))
		return Tango::DEV_DOUBLE;
	
	else if(!strcasecmp(mode, "RE") || !strcasecmp(mode, "REI"))
		return Tango::DEV_DOUBLE;
	else if(!strcasecmp(mode, "RM") || !strcasecmp(mode, "RMI"))
		return Tango::DEV_STRING;
	else if(!strcasecmp(mode, "RS") || !strcasecmp(mode, "RSI") ||
		!strcasecmp(mode, "RAS") || !strcasecmp(mode, "RAIS"))
		return Tango::DEV_SHORT;
        else if(!strcasecmp(mode, "WP") || !strcasecmp(mode, "WPI"))
		return Tango::DEV_VOID;
	else
		return -1;
 }

SpectrumBridgeAttr* AttributeConverter::createSpectrumAttribute(const char* name, 
	long int data_type)
{
		
	UserDefaultAttrProp default_att_prop;
	SpectrumBridgeAttr *spattr;
	char desc[256];
	char res[256];
	char fam[RPCSPLITLEN], mem[RPCSPLITLEN], act[RPCSPLITLEN], mode[RPCSPLITLEN];
	int max_x = MAX_X;

	snprintf(desc, 255, "This attribute maps the RPC call \"%s\" in a tango "
		"READ ONLY spectrum attribute with the same name", name);
	
	if(RPCHandle::split_rpccall(name, fam, mem, act, mode) < 0)
    {
        //printf("\e[1;31m*\e[0m Error in RPC data_type: \"%s\"\n", name);
        exit(EXIT_FAILURE);
    }
	/* particular case in which we know that the RF is long 32 */
	if(strcmp(mode, "RF") == 0)
		max_x = 32;
	
	spattr = new SpectrumBridgeAttr(name, data_type, Tango::READ, max_x);

	if(spattr != NULL)
	{
// 		snprintf(res, 256, "\e[1;32m* \e[0mcreated \"%s\"\t\e[0;37m[\e[0;32mspectrum"
// 		"\e[0;37m, \e[0;32m%s\e[0;37m, \e[0;32mRO\e[0;37m]\e[0m",
//   		name, dataType(data_type).c_str());
        snprintf(res, 256, "created %s spectrum %s", name, dataType(data_type).c_str());
		result = string(res);
	
		default_att_prop.set_description(desc);
		spattr->set_default_properties(default_att_prop);
	}
	return spattr;
}

BridgeAttr* AttributeConverter::createScalarAttribute(const char* name, long int data_type)          
{

    //DEBUG_STREAM << "createScalarAttribute "<< name;
    char desc[256];
	char res[256];
	UserDefaultAttrProp default_att_prop;
	BridgeAttr* attr;
	snprintf(desc, 255, "This attribute maps the RPC call \"%s\" in a tango "
		"READ ONLY scalar attribute with the same name", name);
	attr = new BridgeAttr(name, data_type, Tango::READ);
	
	if(attr != NULL)
	{
// 		snprintf(res, 256, "\e[1;32m* \e[0mcreated \"%s\"\t\e[0;37m[\e[0;32mscalar"
// 		"\e[0;37m, \e[0;32m%s\e[0;37m, \e[0;32mRO\e[0;37m]\e[0m",
//   		name, dataType(data_type).c_str());

        snprintf(res, 256, "created %s scalar  %s", name, dataType(data_type).c_str());

		result = string(res);
		default_att_prop.set_description(desc);
		attr->set_default_properties(default_att_prop);
	}
	return attr;	
}

BridgeAttr* AttributeConverter::createScalarAttribute(const char* name, const char* writeName, long int data_type)
{
    //DEBUG_STREAM << "create ScalarAttribute_write " << name;
    char desc[256];
	char res[256];
	UserDefaultAttrProp default_att_prop;
	BridgeAttr* attr;
	snprintf(desc, 255, "This attribute maps the RPC call \"%s\" in a tango "
		"READ WRITE scalar attribute. RPC read call: \"%s\" write call: \"%s\"", name, name, writeName);
        //printf("createScalarAttribute_write %s %s...\n",name,writeName);
	attr = new BridgeAttr(name, writeName, data_type, Tango::READ_WRITE);
	//printf(".....createScalarAttribute_write %s %s\n",name,writeName);
	if(attr != NULL)
	{
// 		snprintf(res, 256, "\e[1;32m* \e[0mcreated \"%s\"\t\e[0;37m[\e[0;32mscalar"
// 		"\e[0;37m, \e[0;32m%s\e[0;37m, \e[0;32mRW\e[0;37m]\e[0m",
//   		name, dataType(data_type).c_str());
        
        snprintf(res, 256, "created %s scalar %s", name, dataType(data_type).c_str());
        
		default_att_prop.set_description(desc);
		attr->set_default_properties(default_att_prop);
	}
	return attr;	
}

string AttributeConverter::dataType(long int dt)
{
	string s = "";
	switch(dt)
	{
		case Tango::DEV_DOUBLE:
			s += "double";
			break;
		case Tango::DEV_SHORT:
			s += "short";
			break;
		case Tango::DEV_LONG:
			s += "(long) int";
			break;
		case Tango::DEV_STRING:
			s += "string";
			break;
		case Tango::DEV_BOOLEAN:
			s += "boolean";
			break;
		case Tango::DEV_UCHAR:
			s += "unsigned char";
			break;
		case Tango::DEV_USHORT:
			s += "unsigned short";
			break;
                case Tango::DEV_VOID:
			s += "void";
			break;
		default:
			s += "unrecognized";
			break;
	}
	return s;
}

} /* end namespace */

