#ifndef _ATTRIBUTE_CONVERTER_H

#define ATTRIBUTE_CONVERTER_H

#include <BridgeAttr.h>

namespace RPC2TangoBridge_ns
{

class AttributeConverter
{
	public:
		AttributeConverter(char *rpc_call, bool force_spectrum = false);
		
		bool isSpectrum();
		
		BridgeAttr* scalarAttribute() { return sc_attr; }
		SpectrumBridgeAttr *spectrumAttribute() { return sp_attr; }
		string info() { return result; }
		
	private:
		BridgeAttr *sc_attr;
		SpectrumBridgeAttr* sp_attr;
		
		bool string1_contains2(const char* s1, const char* s2);
		bool is_spectrum(const char *call);
		long int data_type_for_mode(const char *mode);

		BridgeAttr* createScalarAttribute(const char* name, long int data_type);
		BridgeAttr* createScalarAttribute(const char* name, const char* writeName, long int data_type);
		SpectrumBridgeAttr* createSpectrumAttribute(const char* name, 
			long int data_type);
		
		/* detects if the entry read from the tango attribute properties contains a ':' separator
	 	* and, if so, extracts and saves in rwname the Write part (the RPC WRITE call.
	 	* rwname must be a pointer to a MAXLEN char vector.
		* Always copies read RPC call in rname.
		* If read write, copies in rwname the write RPC call and returns true, otherwise it 
		* returns false and strlen(rwname) is 0
		*/
		bool isReadWrite(char* entry, char *rname, char *rwname);
		string dataType(long int dt);
		
		bool spectrum;
		string result;
};

} /* namespace */


#endif


