#include <tango.h>
#include <lpc.h>
#include <RPC2TangoBridge.h>
#include <BridgeAttr.h>
#include <RpcHandle.h>

using namespace Tango;
using namespace std;

namespace RPC2TangoBridge_ns
{

void RPC2TangoBridge::read_dynamic_attribute(Tango::Attribute &attr)
{
//  	 printf("\r                                                                    ");
//  	 fflush(stdout);
// 	 printf("\r                                                                  \r");
//  	 printf("\e[1;32m* \e[1;37mreading \e[0m%s\e[0m", attr.get_name().c_str() );
//  	 fflush(stdout);
	
	/* lpc arrays */
	 intArray vresi;
	realArray vresd;
	shortArray sa;
	booleanArray sb;
	charArray sc;
	unsigned char rpcBool;

	long int resi;
	double resd;
	short s;
	unsigned char uc;
	string ress;

	int i;
	int ok;

	char error[256];

	/* search name in scalars map first */
 	map<string, BridgeAttr*>::iterator mapiter;
	mapiter = namesMap.find(attr.get_name());

	if(mapiter != namesMap.end())
 	{
		BridgeAttr *brattr = (*mapiter).second;
		fflush(stdout);
// 		printf("read_dynamic_attribute: name %s\n", brattr->get_name().c_str());
		RPCReadHandle *rpch = brattr->rpcReadHandle();
// 		printf("read_dynamic_attribute: name %s\n", brattr->get_name().c_str());
		if(rpch)
		{
			rpch->lockRpc();
			Tango::WAttribute *attr_set;
			try{
			        Tango::MultiAttribute *attr_list = get_device_attr();
				attr_set = &(attr_list->get_w_attr_by_name(attr.get_name().c_str()));
			}
			catch(DevFailed &e)
			{
				//printf("\e[1;31mfailed to find attribute with name \"%s\"\e[0m\n", 
				//	attr.get_name().c_str());
				attr_set = NULL;
			}
			switch(brattr->get_type())
			{
				case Tango::DEV_LONG:
				resi = rpch->read_long(attr.get_name().c_str(), &ok);
				if(ok)
				{
					DevLong *readVal = new DevLong;
					*readVal = resi;
					attr.set_value(readVal, 1, 0, true);
					if(attr_set)
						attr_set->set_write_value(readVal);
				}
				break;
				case Tango::DEV_DOUBLE:
					resd = rpch->read_double(attr.get_name().c_str(), &ok);
					if(ok)
					{
						DevDouble *readVald = new DevDouble;
						*readVald = resd;

						if(attr_set)
						{
						//	printf("\e[1;32msetto write value a %f\e[0m\n",
						//		*readVald);
							attr_set->set_write_value(readVald);
						}
						attr.set_value(readVald, 1, 0, true);

					}
					break;
				
				case Tango::DEV_SHORT:
					s =  rpch->read_short(attr.get_name().c_str(), &ok);
					if(ok)
					{
						DevShort *readVals = new DevShort;
						*readVals = s;
						attr.set_value(readVals, 1, 0, true);
						if(attr_set)
							attr_set->set_write_value(readVals);

					}
				break;
				
				case Tango::DEV_UCHAR:
					uc =  rpch->read_uchar(attr.get_name().c_str(), &ok);
					if(ok)
					{
						DevUChar *readValc = new DevUChar;
						*readValc =uc;
						attr.set_value(readValc, 1, 0, true);
						if(attr_set)
							attr_set->set_write_value(readValc);

					}
				break;
				
				case Tango::DEV_STRING:
					ress =  rpch->read_string(attr.get_name().c_str(), &ok);
					if(ok)
					{
						DevString *readVals = new DevString;
						*readVals = new char[ress.length() + 1];
						/* be sure it is null terminated */
						memset(*readVals, 0, ress.length() + 1);
						strncpy(*readVals, ress.c_str(), ress.length());
						attr.set_value(readVals, 1, 0, true);
						if(attr_set)
							attr_set->set_write_value(readVals);

					}
				break;
				
				case Tango::DEV_BOOLEAN:
					rpcBool = rpch->read_boolean(attr.get_name().c_str(), &ok);
					if(ok)
					{
						DevBoolean *db = new DevBoolean;
                        sb.length = 1;
						if(rpcBool == 0)
							*db = false;
						else
							*db = true;
						attr.set_value(db, sb.length, 0, true);
						if(attr_set)
							attr_set->set_write_value(db);

					}
					break;
				case Tango::DEV_VOID:
					rpcBool = true;
                                        ok = true; //all dummies
					if(ok)
					{
						DevBoolean *db = new DevBoolean;
                        sb.length = 1;
						if(rpcBool == 0)
							*db = false;
						else
							*db = true;
						attr.set_value(db, sb.length, 0, true);
						if(attr_set)
							attr_set->set_write_value(db);

					}
					break;
				default:
					
					//printf("\t[\e[1;31mfailed\e[0m] (format not supported - %d)\n", brattr->get_type());
					Except::throw_exception("Error reading scalar attribute " + attr.get_name(),
						"The format is not supported.\nSupported formats: long, double",
						"RPC2TangoBridge::read_BridgeAttr()");
					break;
			}
			rpch->unlockRpc();

			if(!ok)
			{
// 				 printf("\r                                                                  \r");
//  				 printf("\e[1;31m* \e[1;37mreading \e[0m%s\e[0m", attr.get_name().c_str() );
// 				printf("\t[\e[1;31mfailed\e[0m]\n\t{%s\n\t\t[host: %s:%d]}\n", rpch->lastRpcError().c_str(), rpch->host().c_str(), rpch->port());

				snprintf(error, 256, "Error reading scalar attribute \"%s\". Error: %s", attr.get_name().c_str(),
						rpch->lastRpcError().c_str());
			//	set_state(Tango::FAULT);
				set_status(error);
				Except::throw_exception("Error reading scalar attribute " + attr.get_name(),
						"RPC call failed: " + rpch->lastRpcError() ,
						"RPC2TangoBridge::read_BridgeAttr()");
				
			}
		}
 	}
	
	/* look among spectrum */
	map<string, SpectrumBridgeAttr*>::iterator spmapiter;
	
	spmapiter = spectrumNamesMap.find(attr.get_name());

	if(spmapiter != spectrumNamesMap.end())
	{
		SpectrumBridgeAttr *spbrattr = (*spmapiter).second;
 		RPCReadHandle *rpch = spbrattr->rpcReadHandle();
		if(rpch)
		{
			rpch->lockRpc();
			switch(spbrattr->get_type())
			{
				case Tango::DEV_LONG:
					vresi = rpch->read_slong(attr.get_name().c_str(), &ok);
					if(ok && vresi.sequence != NULL)
					{
						DevLong *readVal = new DevLong[vresi.length];
						for(i = 0; i < vresi.length; i++)
							readVal[i] = vresi.sequence[i];
						attr.set_value(readVal, vresi.length, 0, true);
						free (vresi.sequence);
					}
					break;
				case Tango::DEV_DOUBLE:
					vresd = rpch->read_sdouble(attr.get_name().c_str(), &ok);
					if(ok && vresd.sequence != NULL)
					{
						DevDouble *readVald = new DevDouble[vresd.length];
						for(i = 0; i < vresd.length; i++)
							readVald[i] = vresd.sequence[i];
						attr.set_value(readVald, vresd.length, 0, true);
						free (vresd.sequence);
					}
					break;
				case Tango::DEV_SHORT:
					sa = rpch->read_sshort(attr.get_name().c_str(), &ok);
					if(ok && sa.sequence != NULL)
					{
						DevShort *ds = new DevShort[sa.length];
						for(i = 0; i < sa.length; i++)
							ds[i] = sa.sequence[i];
						attr.set_value(ds, sa.length, 0, true);
						free (sa.sequence);
					}
					break;
				
				case Tango::DEV_UCHAR:
					sc = rpch->read_schar(attr.get_name().c_str(), &ok);
					if(ok && sc.sequence != NULL)
					{
						DevUChar *dc = new DevUChar[sc.length];
						for(i = 0; i < sc.length; i++)
							dc[i] = sc.sequence[i];
						attr.set_value(dc, sc.length, 0, true);
						free (sc.sequence);
					}
					break;

				case Tango::DEV_BOOLEAN:
					sb = rpch->read_sboolean(attr.get_name().c_str(), &ok);
					if(ok && sb.sequence != NULL)
					{
						DevBoolean *db = new DevBoolean[sb.length];
						for(i = 0; i < sb.length; i++)
							db[i] = sb.sequence[i];
						attr.set_value(db, sb.length, 0, true);
						free (sb.sequence);
					}
					else{
						//printf("\e[1;31m*\e[0m error reading boolean array for \"%s\" : %s\n", attr.get_name().c_str(), rpch->lastRpcError().c_str());
					}
					break;

				default:
					//printf("\t[\e[1;31mfailed\e[0m] (format not supported)\n");
					Except::throw_exception("Error reading spectrum attribute " + attr.get_name(),
						"The format is not supported.\nSupported formats: long, double, uchar, boolean, short",
						"RPC2TangoBridge::read_BridgeAttr()");
					
					break;
			}
			rpch->unlockRpc();

			if(!ok)
			{
// 				 printf("\r                                                                  \r");
//  				 printf("\e[1;31m* \e[1;37mreading \e[0m%s\e[0m", attr.get_name().c_str() );
// 				printf("\t[\e[1;31mfailed\e[0m]\n\t{%s\n\t\t[host: %s:%d]}\n", rpch->lastRpcError().c_str(), rpch->host().c_str(), rpch->port());
				snprintf(error, 256, "Error reading scalar attribute \"%s\". Error: %s", attr.get_name().c_str(),
						rpch->lastRpcError().c_str());
//				set_state(Tango::FAULT);
				set_status(error);

				Except::throw_exception("Error reading spectrum attribute " + attr.get_name(),
					string("RPC call failed: ")  + rpch->lastRpcError(),
					"RPC2TangoBridge::read_BridgeAttr()");
				
			}
			else
			{
				set_state(Tango::ON);
				set_status("server running");
			}
 		}
 	}

}


} /* namespace ends here */


